unit TrayHelperAPI;

interface

uses
  Forms,
  Windows;

type
  TTrayHelperPlugin = class
  private
    FVersion: Real;
    FShortName: String;
    FName: String;
    FOnGetYourName_LineInTxtFile: Integer;
    FOnGetYourShortName_LineInTxtFile: Integer;
    FForm: TForm;
    FParent: HWnd;
    FWndProcLong: integer;
    procedure __CreatePP(APoint: TPoint; AHWnd: Hwnd);              // Required Function
    procedure __OnMovePP(APoint: TPoint);                           // Required Function
    procedure __OnKillActivatePP;                                   // Required Function
    procedure __OnSetActivatePP;                                    // Required Function
    procedure __OnStartYourBackgroundActivityPP;                    // Optional Recommended Function
    procedure __OnPauseAllBackgroundActivityPP;                     // Optional Recommended Function
    procedure __OnResumeAllBackgroundActivityPP;                    // Optional Recommended Function
    function __DoSomething_1PP(APtr: Pointer): Pointer;             // For future implementations
    function __DoSomething_2PP(APtr: Pointer): Pointer;             // For future implementations
    function __DoSomething_3PP(APtr: Pointer): Pointer;             // For future implementations
  protected
    function CreateForm: TForm; virtual;
    procedure OnStartYourBackgroundActivityPP; virtual;           // Optional Recommended Function
    procedure OnPauseAllBackgroundActivityPP; virtual;            // Optional Recommended Function
    procedure OnResumeAllBackgroundActivityPP; virtual;           // Optional Recommended Function
    function DoSomething_1PP(APtr: Pointer): Pointer; virtual;    // For future implementations
    function DoSomething_2PP(APtr: Pointer): Pointer; virtual;    // For future implementations
    function DoSomething_3PP(APtr: Pointer): Pointer; virtual;    // For future implementations
    property Version: Real read FVersion write FVersion;
    property Name: String read FName write FName;
    property ShortName: String read FShortName write FShortName;
    property OnGetYourName_LineInTxtFile: Integer read FOnGetYourName_LineInTxtFile write FOnGetYourName_LineInTxtFile;
    property OnGetYourShortName_LineInTxtFile: Integer read FOnGetYourShortName_LineInTxtFile write FOnGetYourShortName_LineInTxtFile;
  public
    destructor destroy; override;
  end;

procedure StartTrayHelperPlugin(ATrayHelperPlugin: TTrayHelperPlugin; AIconResourceName: String = '');

implementation

uses
  Messages,
  SysUtils;

type
  PHWnd = ^HWnd;

var
  gTrayHelperPlugin: TTrayHelperPlugin = nil;
  gIconResourceName: String = '';

procedure CreatePP(APoint: TPoint; AHWnd: PHwnd); cdecl;
begin
  if assigned(gTrayHelperPlugin) then
    begin
    gTrayHelperPlugin.__CreatePP(APoint,AHwnd^);
    end;
end;

procedure OnMovePP(APoint: TPoint); cdecl;
begin
  if assigned(gTrayHelperPlugin) then
    begin
    gTrayHelperPlugin.__OnMovePP(APoint);
    end;
end;

procedure OnKillActivatePP; cdecl;
begin
  if assigned(gTrayHelperPlugin) then
    begin
    gTrayHelperPlugin.__OnKillActivatePP;
    end;
end;

procedure OnSetActivatePP; cdecl;
begin
  if assigned(gTrayHelperPlugin) then
    begin
    gTrayHelperPlugin.__OnSetActivatePP;
    end;
end;

function OnGetYourNamePP: PChar; cdecl;
begin
  result := nil;
  if assigned(gTrayHelperPlugin) then
    begin
    result := PChar(gTrayHelperPlugin.Name);
    end;
end;

procedure OnExitPP; cdecl;
begin
    FreeAndNil(gTrayHelperPlugin);
end;

procedure OnStartYourBackgroundActivityPP; cdecl;
begin
  if assigned(gTrayHelperPlugin) then
    begin
    gTrayHelperPlugin.__OnStartYourBackgroundActivityPP;
    end;
end;

procedure OnPauseAllBackgroundActivityPP; cdecl;
begin
  if assigned(gTrayHelperPlugin) then
    begin
    gTrayHelperPlugin.__OnPauseAllBackgroundActivityPP;
    end;
end;

procedure OnResumeAllBackgroundActivityPP; cdecl;
begin
  if assigned(gTrayHelperPlugin) then
    begin
    gTrayHelperPlugin.__OnResumeAllBackgroundActivityPP;
    end;
end;

function OnGetYourName_LineInTxtFile: Integer; cdecl;
begin
  result := 0;
  if assigned(gTrayHelperPlugin) then
    begin
    result := gTrayHelperPlugin.OnGetYourName_LineInTxtFile;
    end;
end;

function OnGetYourShortName_LineInTxtFile: Integer; cdecl;
begin
  result := 0;
  if assigned(gTrayHelperPlugin) then
    begin
    result := gTrayHelperPlugin.OnGetYourShortName_LineInTxtFile;
    end;
end;

function OnGetYour_Version: Real; cdecl;
begin
  result := 0.0;
  if assigned(gTrayHelperPlugin) then
    begin
    result := gTrayHelperPlugin.Version;
    end;
end;

function DoSomething_1PP(APtr: Pointer): Pointer; cdecl;
begin
  result := nil;
  if assigned(gTrayHelperPlugin) then
    begin
    result := gTrayHelperPlugin.__DoSomething_1PP(APtr);
    end;
end;

function DoSomething_2PP(APtr: Pointer): Pointer; cdecl;
begin
  result := nil;
  if assigned(gTrayHelperPlugin) then
    begin
    result := gTrayHelperPlugin.__DoSomething_2PP(APtr);
    end;
end;

function DoSomething_3PP(APtr: Pointer): Pointer; cdecl;
begin
  result := nil;
  if assigned(gTrayHelperPlugin) then
    begin
    result := gTrayHelperPlugin.__DoSomething_3PP(APtr);
    end;
end;

function OnGetYourIconPP: HICON; cdecl;
begin
  if gIconResourceName = '' then
    begin
    result := 0;
    end
  else
    begin
    result := LoadIcon(HInstance,PChar(gIconResourceName));
    end;
end;

procedure StartTrayHelperPlugin(ATrayHelperPlugin: TTrayHelperPlugin; AIconResourceName: String);
begin
  gIconResourceName := AIconResourceName;
  Assert(not Assigned(gTrayHelperPlugin),'gTrayHelperPlugin has already been started');
  gTrayHelperPlugin := ATrayHelperPlugin;
end;

{ TTrayHelperPlugin }

function TTrayHelperPlugin.CreateForm: TForm;
begin
  result := nil;
end;

function HideChildren(AWnd: HWnd; AParam: Integer): Bool; stdcall;
begin
  result := true;
  ShowWindow(AWnd,SW_HIDE);
end;

destructor TTrayHelperPlugin.destroy;
begin
  SetWindowLong(FParent,GWL_WNDPROC,FWndProcLong);  // put it back the way it was
  FreeAndNil(FForm);
  inherited;
end;

function WndProc(AWnd: HWND; uMsg: UINT; AwParam: WPARAM; AlParam: LPARAM): LRESULT; stdcall;
var
  pp: TPoint;   // just a dummy
begin
  result := CallWindowProc(pointer(gTrayHelperPlugin.FWndProcLong),AWnd,uMsg,AwParam,AlParam);
  if uMsg = WM_SIZE then
    begin
    gTrayHelperPlugin.__OnMovePP(pp);   // resize etc the window
    end;
end;

function TTrayHelperPlugin.DoSomething_1PP(APtr: Pointer): Pointer;
begin
  result := nil;
end;

function TTrayHelperPlugin.DoSomething_2PP(APtr: Pointer): Pointer;
begin
  result := nil;
end;

function TTrayHelperPlugin.DoSomething_3PP(APtr: Pointer): Pointer;
begin
  result := nil;
end;

procedure TTrayHelperPlugin.OnPauseAllBackgroundActivityPP;
begin
  // empty
end;

procedure TTrayHelperPlugin.OnResumeAllBackgroundActivityPP;
begin
  // empty
end;

procedure TTrayHelperPlugin.OnStartYourBackgroundActivityPP;
begin
  // empty
end;

procedure TTrayHelperPlugin.__CreatePP(APoint: TPoint; AHWnd: Hwnd);
begin
  EnumChildWindows(AHWnd,@HideChildren,0);
  FForm := CreateForm;
  FForm.BorderStyle := bsNone;
  FParent := AHWnd;
  FForm.ParentWindow := FParent;
  FForm.Show;
  FWndProcLong := GetWindowLong(AHWnd,GWL_WNDPROC);
  SetWindowLong(AHWnd,GWL_WNDPROC,integer(@WndProc));
  __OnMovePP(APoint);
end;

function TTrayHelperPlugin.__DoSomething_1PP(APtr: Pointer): Pointer;
begin
  result := DoSomething_1PP(APtr);
end;

function TTrayHelperPlugin.__DoSomething_2PP(APtr: Pointer): Pointer;
begin
  result := DoSomething_2PP(APtr);
end;

function TTrayHelperPlugin.__DoSomething_3PP(APtr: Pointer): Pointer;
begin
  result := DoSomething_3PP(APtr);
end;

procedure TTrayHelperPlugin.__OnKillActivatePP;
begin
  if Assigned(FForm) then
    begin
    FForm.Hide;
    end;
end;

procedure TTrayHelperPlugin.__OnMovePP(APoint: TPoint);
var
  LWinInfo: TagWindowInfo;
begin
  if not Assigned(FForm) then
    begin
    exit;
    end;
  FForm.Top := 0;
  FForm.Left := 0;
  GetWindowInfo(FParent,LWinInfo);
  FForm.Height := LWinInfo.rcClient.Bottom - LWinInfo.rcClient.Top;
  FForm.Width := LWinInfo.rcClient.Right - LWinInfo.rcClient.Left;
  FForm.Invalidate;
  FForm.Repaint;
  Application.ProcessMessages;
end;

procedure TTrayHelperPlugin.__OnPauseAllBackgroundActivityPP;
begin
  OnPauseAllBackgroundActivityPP;
end;

procedure TTrayHelperPlugin.__OnResumeAllBackgroundActivityPP;
begin
  OnResumeAllBackgroundActivityPP;
end;

procedure TTrayHelperPlugin.__OnSetActivatePP;
begin
  if Assigned(FForm) then
    begin
    FForm.Show;
    end;
end;

procedure TTrayHelperPlugin.__OnStartYourBackgroundActivityPP;
begin
  OnStartYourBackgroundActivityPP;
end;

exports
  CreatePP,
  OnMovePP,
  OnKillActivatePP,
  OnSetActivatePP,
  OnGetYourNamePP,
  OnExitPP,
  OnStartYourBackgroundActivityPP,
  OnPauseAllBackgroundActivityPP,
  OnResumeAllBackgroundActivityPP,
  OnGetYourName_LineInTxtFile,
  OnGetYourShortName_LineInTxtFile,
  OnGetYour_Version,
  DoSomething_1PP,
  DoSomething_2PP,
  DoSomething_3PP,
  OnGetYourIconPP;

end.
